LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

ENTITY ImProE IS
  PORT(
    Button   : IN  std_logic;
    Clk12    : IN  std_logic;
    Reset    : IN  std_logic;
    PixR     : IN  std_logic_vector (11 DOWNTO 0);
    WeA      : OUT std_logic_vector (0 DOWNTO 0);
    Mode     : OUT std_logic_vector (3 DOWNTO 0);
    PixE     : OUT std_logic_vector (11 DOWNTO 0);
    ResetAll : OUT std_logic
    );

END ImProE;

ARCHITECTURE ImProA OF ImProE IS
	
	COMPONENT ImConE IS
		PORT (Button    : IN  std_logic;
           Clk12    : IN  std_logic;
           Reset    : IN  std_logic;
           Mode     : OUT std_logic_vector (3 DOWNTO 0);
           ResetAll : OUT std_logic); 
	END COMPONENT;

	Component ImEditE
		Port(	PixR: in std_logic_vector(11 downto 0);	
				Mode: in std_logic_vector(3 downto 0);
				WeA: out std_logic_vector(0 downto 0);
				PixE: out std_logic_vector(11 downto 0)
				);
	END Component;

	signal ModeS : std_logic_vector(3 downto 0);

BEGIN
	
	ImCon:ImConE
		port map (
		Button 	=> Button,
		Clk12 	=> Clk12,
		Reset 	=> Reset,
		Mode 		=> ModeS,
		ResetAll	=> ResetAll
		);
		
	ImEdit:ImEditE
		port map (
		PixR		=> PixR,
		Mode		=>	ModeS,
		WeA		=>	WeA,
		PixE		=> PixE
		);
		
	Mode <= ModeS;
		
END ImProA;

-- pragma synthesis_off 
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_signed.ALL;
USE ieee.std_logic_arith.ALL;

ENTITY ImProTbE IS
  GENERIC (ClkT : time := 83.33 ns);
END ImProTbE;

ARCHITECTURE ImProTbA OF ImProTbE IS
  
  COMPONENT ImProE IS
    PORT(
      Button   : IN  std_logic;
      Clk12    : IN  std_logic;
      Reset    : IN  std_logic;
      PixR     : IN  std_logic_vector (11 DOWNTO 0);
      WeA      : OUT std_logic_vector (0 DOWNTO 0);
      Mode     : OUT std_logic_vector (3 DOWNTO 0);
      PixE     : OUT std_logic_vector (11 DOWNTO 0);
      ResetAll : OUT std_logic
      ); 
  END COMPONENT;

  SIGNAL ButtonS, Clk12S, ResetS, ResetAllS : std_logic := '0';
  SIGNAL ModeS                              : std_logic_vector (3 DOWNTO 0);
  SIGNAL PixRS                              : std_logic_vector (11 DOWNTO 0);
  SIGNAL WeAS                               : std_logic_vector (0 DOWNTO 0);
  SIGNAL PixES                              : std_logic_vector (11 DOWNTO 0);

  SIGNAL stopS : std_logic := '0';

BEGIN
  
  ImProL : ImProE
    PORT MAP (Button    => ButtonS,
               Clk12    => Clk12S,
               Reset    => ResetS,
               PixR     => PixRS,
               WeA      => WeAS,
               Mode     => ModeS,
               PixE     => PixES,
               ResetAll => ResetAllS);   

-- Takt 
  PROCESS
  BEGIN
    Clk12S <= '0';
    WAIT FOR 41.667 ns;
    Clk12S <= '1';
    WAIT FOR 41.667 ns;
  END PROCESS;

  ResetS <= '0' AFTER 0*ClkT,
            '1' AFTER 0.5*ClkT;
  
  ButtonS <= '1' AFTER 0*ClkT,
             '0' AFTER 2*ClkT,
             '1' AFTER 10*ClkT,
             '0' AFTER 12*ClkT,
             '1' AFTER 15*ClkT,
             '0' AFTER 18*ClkT,
             '1' AFTER 21*ClkT,
             '0' AFTER 24*ClkT,
             '1' AFTER 30*ClkT;
  PixRS <= "111111111111";
  stopS <= '0' AFTER 0*ClkT,
           '1' AFTER 50*ClkT;
--assert now<40*ClkT                            
  ASSERT stopS = '0'
    REPORT "stop simulation"
    SEVERITY failure;

END ImProTbA;

-- pragma synthesis_on

